set SSS_PROG "galileo"

#################### Global variables linked to C ######################
# running - whether or not a run is currently in progress
# graphX, graphY - "Time", "Theta", "ThetaDot", "Energy", "KineticEnergy",
#		"PotentialEnergy", or "Torque"

# galileo parameters
# Theta0, ThetaDot0, t0 - Initial conditions

# Debugging Tool
# LASSPinterp .interp	

set Theta0 0.7
set ThetaDot0 0
set t0 0

set L 10			;# 10 meter pendulum 
set Mass 0.05			;# 50 gm = 0.05 kg 
set g 9.81			;# m/s^2 
set alpha 0			;# Damping constant 

# Runge-Kutta Parameters

set dt 0.05;			;# Step size for Runge Kutta 

# Animation parameters

set v_mult 1.0			;# Velocity per Meter for Arrow on Animation 
set Bob_color "green"
set Stick_color "brown"
set Bob_radius 0.8
set Stick_thickness 0.4
# Size to see pendulum swing in box: aspect ratio kept fixed
set size_view [expr $L + $Bob_radius]

# Graph Parameters

set graphX "Time"			;# Plot Theta vs Time default 
set graphY "Theta"
set user_name "Waldo"			;# User's name

# Communication with C Program

set running 0			;# Zero to stop animation, one while running 
set dynamics_type "exact"

################### Global variables Not linked to C ####################
# num_graphs - used for the LASSPgraph routines

set num_graphs 0
set zooming 0			 ;# To tell if animation is currently zooming

################################## center controls  ############
frame .c

proc stop {} {
    global tmax_store running .tM
    if {$running} {
	.run configure -text "Run"
	LASSPscale2_set .tM $tmax_store
	set running 0
    }
}

button .run -text "Run" -width 10 -command {
    if {$running} {stop} else {
	.run configure -text "Running"
	update
	set running 1
	set tmax_store $tmax
	Run 0 $tmax
	if {$running} {		;# Leave .tM partial if stopped in middle
	    LASSPscale2_set .tM $tmax_store
	    .run configure -text "Run"
	    set running 0
	    update
	}
    }
}

LASSPscale2 .theta0 -text "Theta Initial (radians): " -scrollmin -4 -scrollmax 4 \
	-initial 0.7 -scrollinc 0.01 -type real \
	-var Theta0 -command {reset_to_initial}

LASSPscale2 .thetaDot0 -text "Theta Dot Initial (rad/sec): " \
	-scrollmin -10 -scrollmax 10 \
	-initial 0 -scrollinc 0.01 -type real -var ThetaDot0 -command {reset_to_initial}

LASSPscale2 .tM -text "Time to Run: (sec) " -scrollmin 0 -scrollmax 100 \
	-initial 10 -scrollinc 0.1 -type real -var tmax \
	-test {%v>0}

LASSPscale2 .alpha -text "Damping alpha (1/sec): " -scrollmin 0 -scrollmax 10 \
	-initial 0 -scrollinc 0.1 -type real -var alpha

# LASSPpresets .presets presetsave
button .configure -text "Configure" -command {configure}
button .help -text "Assignment, and Help" -command {LASSPhelp2 .t2 [file join $SSS_DIR lib/galileo/galileo.htm]}
button .quit -text "Quit" -command {exit}

pack .run -in .c -fill x -pady 2
pack .theta0 -in .c -fill x -pady 2
pack .thetaDot0 -in .c -fill x -pady 2
pack .alpha -in .c -fill x -pady 2
pack .tM -in .c -fill x -pady 2
# pack .presets -in .c -fill x 
pack .configure -in .c -fill x 
pack .help -in .c -fill x 
pack .quit -in .c -fill x 
wm minsize . 200 200

################# graph controls ##############################################
frame .g
frame .gmenu
button .copygraph -text "Copy Graph" -command {
    incr num_graphs 1
    graph_create .graph$num_graphs .graph 
    .graph$num_graphs.graph configure -title $user_name
    .graph$num_graphs.graph psconfigure -landscape true \
	-pageheight 9i -pagewidth 6.5i
}
frame .graphtype
    LASSPselection .graphtype.graphY {} graphY {Pendulum_data_display} \
	"Theta" "ThetaDot" "Energy" "Time" \
	"PotEngy" "KinEngy" "Torque"
    label .graphtype.vs -text "vs."
    LASSPselection .graphtype.graphX {} graphX {Pendulum_data_display} \
	"Time" "Theta" "ThetaDot" "Energy" \
	"PotEngy" "KinEngy" "Torque"
    pack .graphtype.graphY -side left
    pack .graphtype.vs -side left
    pack .graphtype.graphX -side left

pack .copygraph -in .gmenu -side left -padx 2
pack .graphtype -in .gmenu -side right -padx 2
pack .gmenu -in .g -side top -fill x -pady 1

################### Animation controls ########################################
frame .a
frame .amenu

proc initialize_animation {} {
    global running L Theta0 center_x center_y expand
    global Bob_color Bob_radius Stick_color Stick_thickness

    if {$running} {} else {

	set x [expr $L*sin($Theta0)]
	set y [expr $L*cos($Theta0)]	;# Upside down convention for X-windows

	.pendulum delete Bob
	.pendulum delete Stick

	.pendulum create line \
	    $center_x $center_y \
	    [expr $expand*$x+$center_x] [expr $expand*$y+$center_y] \
	    -fill $Stick_color -width [expr $expand*$Stick_thickness] \
	    -tags Stick

	.pendulum create oval \
	    [expr $expand*($x-$Bob_radius)+$center_x] \
	    [expr $expand*($y-$Bob_radius)+$center_y] \
	    [expr $expand*($x+$Bob_radius)+$center_x] \
	    [expr $expand*($y+$Bob_radius)+$center_y] \
	    -fill $Bob_color -width 0 -tags Bob

    }
}

proc reset_to_initial {} {
    global t0
    global Theta0 ThetaDot0
    global Mass g L

    stop
    .run configure -text "Run"

    initialize_animation
}


LASSPselection .dynamics {} dynamics_type {Pendulum_switch_dynamics} \
	"exact" "approx"

frame .user_name
    label .user_name.l -text "Your Name:"
    LASSPentry .user_name.e  {catch {set user_name [%W get]}}
    LASSPentry_set .user_name.e $user_name
    pack .user_name.l -side left
    pack .user_name.e -side left -fill x -expand yes

pack .dynamics .user_name -in .amenu -side left -padx 2
pack .amenu -in .a -side top -fill x

################### the animation window ######################################
canvas .pendulum -width 200 -height 200 -relief sunken -background black -bd 2

# Zooming stuff, for galileo
# Zoom with Control-Button-1 in canvas window

# center in pixel coordinates:
set center_x [expr double([winfo reqwidth .pendulum])/2.]
set center_y [expr double([winfo reqheight .pendulum])/2.]
# expand is expansion factor Length -> Pixels
# view_left, view_top is upper left hand
# view_right, view_bottom is lower right
if {$center_x > $center_y} {
    set expand [expr $center_y/$size_view]
    set view_left [expr -($center_x/$center_y)*$size_view]
    set view_right [expr ($center_x/$center_y)*$size_view]
    set view_top [expr -$size_view]   ;# Top is minimum y
    set view_bottom [expr $size_view]
} else {
    set expand [expr $center_x/$size_view]
    set view_left [expr -$size_view]
    set view_right [expr $size_view]
    set view_top [expr -($center_y/$center_x)*$size_view]
    set view_bottom [expr ($center_y/$center_x)*$size_view]
}

# Setup Zoom
proc setup_zoom {x y} {
    global .pendulum band_xI band_yI band_x1 band_y1 band_x2 band_y2
    global zooming

    .pendulum create rectangle $x $y $x $y -tags band -outline white
    set band_xI $x
    set band_yI $y
    set band_x1 $x
    set band_y1 $y
    set band_x2 $x
    set band_y2 $y

    set zooming 1
}

bind .pendulum <Control-ButtonPress-1> {
    setup_zoom %x %y
}
bind .pendulum <ButtonPress-3> {
    setup_zoom %x %y
}

# Expand zoom rubber band
proc zoom_band {x y} {
    global band_xF band_yF band_xI band_yI band_x1 band_y1 band_x2 band_y2
    global .pendulum

    set band_xF $x
    set band_yF $y
    if {$band_xF > $band_xI} {
        set band_x1 $band_xI; set band_x2 $band_xF
    } else {
        set band_x1 $band_xF; set band_x2 $band_xI
    }
    if {$band_yF > $band_yI} {
        set band_y1 $band_yI; set band_y2 $band_yF
    } else {
        set band_y1 $band_yF; set band_y2 $band_yI
    }
    .pendulum coords band $band_x1 $band_y1 $band_x2 $band_y2

}

bind .pendulum <Control-Button1-Motion> {
    zoom_band %x %y
}
bind .pendulum <Button3-Motion> {
    zoom_band %x %y
}


# Reset to original scale
proc reset_scale {} {
    global size_view expand center_x center_y
    set win_x [expr double([winfo width .pendulum])]
    set win_y [expr double([winfo height .pendulum])]
    if {$win_x > $win_y} {
        set A [expr $win_y/(2.*$size_view*$expand)]
    } else {
        set A [expr $win_x/(2.*$size_view*$expand)]
    }
    if {$A == 1} {
        set shift_x [expr ($win_x/2. - $center_x)]
        set shift_y [expr ($win_y/2. - $center_y)]
        .pendulum move all $shift_x $shift_y
        set center_x [expr $center_x + $shift_x]
        set center_y [expr $center_y + $shift_y]
    } else {
        set expand [expr $expand * $A]
        set x_expand [expr ($win_x/2. - $A*$center_x)/(1.-$A)]
        set y_expand [expr ($win_y/2. - $A*$center_y)/(1.-$A)]
        .pendulum scale all $x_expand $y_expand $A $A
        set center_x [expr $A*($center_x - $x_expand) + $x_expand]
        set center_y [expr $A*($center_y - $y_expand) + $y_expand]
    }
    set view_left [expr -$center_x/$expand]
    set view_right [expr ($win_x-$center_x)/$expand]
    set view_top [expr -$center_y/$expand]
    set view_bottom [expr ($win_y-$center_y)/$expand]
}

# Do zooming
proc do_zoom {} {
    global band band_x1 band_y1 band_x2 band_y2
    global .pendulum
    global win_x win_y center_x center_y expand
    global view_left view_right view_top view_bottom
    global zooming

    set win_x [expr double([winfo width .pendulum])]
    set win_y [expr double([winfo height .pendulum])]
    if {[expr ($band_x2 - $band_x1 == 0) || ($band_y2 - $band_y1 == 0)]} {
#
# Put back at starting size
#
        reset_scale
        .pendulum delete band
        initialize_animation
    } else {
#
# Blow up keeping aspect ratios: rescale by whatever's smaller
# Make center of window equal center of zoom box
#
        set Ax [expr $win_x/($band_x2 - $band_x1)]
        set Ay [expr $win_y/($band_y2 - $band_y1)]
        if {$Ax < $Ay} {set A $Ax} else {set A $Ay}
# Either move (if scale unchanged)
        if {$A == 1} {
            set shift_x [expr ($win_x - $band_x1 - $band_x2)/2.]
            set shift_y [expr ($win_y - $band_y1 - $band_y2)/2.]
            .pendulum move all $shift_x $shift_y
            set center_x [expr $center_x + $shift_x]
            set center_y [expr $center_y + $shift_y]
            .pendulum delete band
        } else {
# or scale by A, keeping point (x_expand, y_expand) fixed
            set expand [expr $expand * $A]
            set x_expand [expr ($win_x/2.-$A*($band_x1+$band_x2)/2.)/(1.-$A)]
            set y_expand [expr ($win_y/2.-$A*($band_y1+$band_y2)/2.)/(1.-$A)]
            .pendulum scale all $x_expand $y_expand $A $A
            set center_x [expr $A*($center_x - $x_expand) + $x_expand]
            set center_y [expr $A*($center_y - $y_expand) + $y_expand]
            .pendulum delete band
        }
        set view_left [expr -$center_x/$expand]
        set view_right [expr ($win_x-$center_x)/$expand]
        set view_top [expr -$center_y/$expand]
        set view_bottom [expr ($win_y-$center_y)/$expand]
	initialize_animation
    }

    set zooming 0
}

bind .pendulum <Control-ButtonRelease-1> {
    do_zoom
}

bind .pendulum <ButtonRelease-3> {
    do_zoom
}



bind .pendulum <Configure> {

    set win_x [expr double([winfo width .pendulum])]
    set win_y [expr double([winfo height .pendulum])]

    set expand_old $expand
    set expand_x [expr $win_x/($view_right - $view_left)]
    set expand_y [expr $win_y/($view_bottom - $view_top)]
    if {$expand_x > $expand_y} {
        set expand $expand_y
    } else {
        set expand $expand_x
    }
    set A [expr $expand/$expand_old]
    if {$A == 1} {
        if {$expand_x != $expand_y} {   ;# Equal -> No change???
            set shift_x [expr ($win_x/2.)*(1-$expand/$expand_x)]
            set shift_y [expr ($win_y/2.)*(1-$expand/$expand_y)]
            .pendulum move all $shift_x $shift_y
            set center_x [expr $center_x + $shift_x]
            set center_y [expr $center_y + $shift_y]
        }
    } else {
        if {$expand_x > $expand_y} {
            set right [expr $win_x/2+$A*$expand_old*($view_right-$view_left)/2]
            set bottom $win_y
        } else {
            set right $win_x
            set bottom [expr $win_y/2+$A*$expand_old*($view_bottom-$view_top)/2]
        }
        set x_expand [expr ($right - $A*($expand_old*$view_right+$center_x)) \
                                /(1.-$A)]
        set y_expand [expr ($bottom - $A*($expand_old*$view_bottom+$center_y)) \
                                /(1.-$A)]
        .pendulum scale all $x_expand $y_expand $A $A
        set center_x [expr $A*($center_x - $x_expand) + $x_expand]
        set center_y [expr $A*($center_y - $y_expand) + $y_expand]
    }
    set view_left [expr -$center_x/$expand]
    set view_right [expr ($win_x-$center_x)/$expand]
    set view_top [expr -$center_y/$expand]
    set view_bottom [expr ($win_y-$center_y)/$expand]

}

########################## Select Initial ############################
# Uses button 1 to select initial conditions for various display options #

bind .pendulum <Button1-Motion> {
        # Velocity is component of displacement tangent to circle
        set displacement_x [expr %x - $xI]
        set displacement_y [expr %y - $yI]
        set dot_prod [expr $displacement_x * sin($thetaNew) \
                +  $displacement_y * cos($thetaNew)]
        set velocity_x [expr ($displacement_x - $dot_prod*sin($thetaNew))]
        set velocity_y [expr ($displacement_y - $dot_prod*cos($thetaNew))]
        set thetaDotNew [expr (cos($thetaNew)*$velocity_x \
                                - sin($thetaNew)*$velocity_y) \
                                         /($L*$expand*$v_mult)]
        set xF [expr $xI + $velocity_x]
        set yF [expr $yI + $velocity_y]
        set cor [lreplace [.pendulum coords velocity] 2 3 $xF $yF]
        eval ".pendulum coords velocity $cor"
}
bind .pendulum <ButtonRelease-1> {

# ButtonRelease 1 might be zooming: control button released too soon
    if {$zooming} { do_zoom } else {
	.pendulum delete velocity
	LASSPscale2_set .theta0 $thetaNew
	LASSPscale2_set .thetaDot0 $thetaDotNew
	reset_to_initial
	initialize_animation
	.run invoke
    }
}

bind .pendulum <ButtonPress-1> {
        set thetaNew [expr atan2(%x-$center_x,(%y-$center_y))]
        set thetaDotNew 0
        set xI [expr $L*sin($thetaNew)*$expand+$center_x]
        set yI [expr $L*cos($thetaNew)*$expand+$center_y]
        .pendulum create line $xI $yI $xI $yI -tags velocity \
                        -arrow last -fill orange
}




################### the graph window ######################################

blt_graph .graph -width 300 -height 300 -relief raised -bd 2 \
	-plotbackground white
.graph element create data
.graph element configure data -fg blue
.graph legend configure -mapped no
.graph yaxis configure -title "Theta (radians)"
.graph xaxis configure -title "Time (s)"
LASSPgrzoom .graph
LASSPgrcursor .graph "format {%.4lf,   %.4lf} "




################## Placing the pieces ###################################
pack .graph -in .g -side top -fill both -expand yes 
pack .pendulum -in .a -side top -fill both -expand yes
pack .g -side right -fill both -expand yes -padx 5 -pady 5
pack .c -side right -fill y -padx 5 -pady 5
pack .a -side right -fill both -expand yes -padx 5 -pady 5
wm minsize . 200 200

################### Drawing ####

reset_to_initial


################################## the presetsave procedure ############
proc presetsave {file} {
    #global variables
    foreach s {} {
        global $s
        puts $file "set $s [set $s]"
    }

    #selections
    foreach s {.graphtype} {
        set val [LASSPselection_set $s]
        puts $file "LASSPselection_set $s \"$val\""
    }

    #sliders
    foreach s {.theta0 .thetaDot0 .tM .alpha} {
        set val [LASSPscale2_configure $s -scrollinc]
        puts $file "LASSPscale2_configure $s -scrollinc $val"
        set val [LASSPscale2_configure $s -scrollmin]
        puts $file "LASSPscale2_configure $s -scrollmin $val"
        set val [LASSPscale2_configure $s -scrollmax]
        puts $file "LASSPscale2_configure $s -scrollmax $val"
        set val [LASSPscale2_configure $s -scrolltype]
        puts $file "LASSPscale2_configure $s -scrolltype $val"
        set val [LASSPscale2_configure $s -font]
        puts $file "LASSPscale2_configure $s -font $val"

	set val [LASSPscale2_get $s]
        puts $file "LASSPscale2_set -nocommand $s $val"
    } 
    
}

################################## create a configure window ############
proc configure {} {

    global t0, dt
    global tcl_precision
    global Theta0 ThetaDot0 Bob_radius Stick_thickness
    global Theta_default ThetaDot_default
    global smooth error_max dxsav steps v_mult
    global Mass L tmax
    global Bob_color Stick_color
    global g


# Reset toplevel windows if they exist
    if {[winfo exists .pendulum_configure]} {
    } else {
        toplevel .pendulum_configure
        wm title .pendulum_configure "galileo Configure"
    }

    set w .pendulum_configure

    frame $w.controls
    label $w.controls.l -text "galileo Configure"
    button $w.controls.b -text Close -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.colors
        label $w.colors.l -text "galileo Colors"
        pack $w.colors.l -fill x -expand yes

        frame $w.colors.bob
        label $w.colors.bob.l -text "Bob Color:"
        LASSPentry $w.colors.bob.e  {catch {set Bob_color [%W get]}}
        LASSPentry_set $w.colors.bob.e $Bob_color
        pack $w.colors.bob.l -side left
        pack $w.colors.bob.e -side left -fill x -expand yes
        pack $w.colors.bob -fill x -expand yes -pady 1

        frame $w.colors.stick
        label $w.colors.stick.l -text "Stick Color:"
        LASSPentry $w.colors.stick.e  {catch {set Stick_color [%W get]}}
        LASSPentry_set $w.colors.stick.e $Stick_color
        pack $w.colors.stick.l -side left
        pack $w.colors.stick.e -side left -fill x -expand yes
        pack $w.colors.stick -fill x -expand yes -pady 1

    pack $w.colors -fill x -expand yes -pady 1

    frame $w.sizes
        label $w.sizes.l -text "galileo Sizes"
        pack $w.sizes.l -fill x -expand yes

        frame $w.sizes.bob
        label $w.sizes.bob.l -text "Bob Size (m):"
        LASSPentry $w.sizes.bob.e  {catch {set Bob_radius [%W get]}}
        LASSPentry_set $w.sizes.bob.e  $Bob_radius
        pack $w.sizes.bob.l -side left
        pack $w.sizes.bob.e -side left -fill x -expand yes
        pack $w.sizes.bob -fill x -expand yes -pady 1

        frame $w.sizes.stick
        label $w.sizes.stick.l -text "Stick Thickness (m):"
        LASSPentry $w.sizes.stick.e  {catch {set Stick_thickness [%W get]}}
        LASSPentry_set $w.sizes.stick.e $Stick_thickness
        pack $w.sizes.stick.l -side left
        pack $w.sizes.stick.e -side left -fill x -expand yes
        pack $w.sizes.stick -fill x -expand yes -pady 1

        frame $w.sizes.length
        label $w.sizes.length.l -text "Length L (m):"
        LASSPentry $w.sizes.length.e  {catch {set L [%W get]}}
        LASSPentry_set $w.sizes.length.e $L
        pack $w.sizes.length.l -side left
        pack $w.sizes.length.e -side left -fill x -expand yes
        pack $w.sizes.length -fill x -expand yes -pady 1

    pack $w.sizes -fill x -expand yes -pady 1

    frame $w.initial_conditions
        label $w.initial_conditions.l -text "Initial Conditions"
        pack $w.initial_conditions.l -fill x -expand yes

        frame $w.initial_conditions.theta0
        label $w.initial_conditions.theta0.l -text "Initial Theta (radians):"
        LASSPentry $w.initial_conditions.theta0.e  \
                {catch {set Theta0 [%W get];
		 LASSPscale2_set .theta0 $Theta0;
		 reset_to_initial}}
        LASSPentry_set $w.initial_conditions.theta0.e $Theta0
        pack $w.initial_conditions.theta0.l -side left
        pack $w.initial_conditions.theta0.e -side left -fill x -expand yes
        pack $w.initial_conditions.theta0 -fill x -expand yes -pady 1

       frame $w.initial_conditions.thetaDot0
        label $w.initial_conditions.thetaDot0.l \
		 -text "Initial Theta Dot (rad/sec):"
        LASSPentry $w.initial_conditions.thetaDot0.e  \
                {catch {set ThetaDot0 [%W get];
		 LASSPscale2_set .thetaDot0 $ThetaDot0;
		 reset_to_initial}}
        LASSPentry_set $w.initial_conditions.thetaDot0.e $ThetaDot0
        pack $w.initial_conditions.thetaDot0.l -side left
        pack $w.initial_conditions.thetaDot0.e -side left -fill x -expand yes
        pack $w.initial_conditions.thetaDot0 -fill x -expand yes -pady 1
 
    pack $w.initial_conditions -fill x -expand yes -pady 1

    frame $w.masses
        label $w.masses.l -text "Mass and g"
        pack $w.masses.l -fill x -expand yes

        frame $w.masses.mass
        label $w.masses.mass.l -text "Mass (kg): (irrelevant)"
        LASSPentry $w.masses.mass.e {catch {set Mass [%W get]; reset_to_initial}}
        LASSPentry_set $w.masses.mass.e $Mass
        pack $w.masses.mass.l -side left
        pack $w.masses.mass.e -side left -fill x -expand yes
        pack $w.masses.mass -fill x -expand yes -pady 1

        frame $w.masses.g
        label $w.masses.g.l -text "Gravitational Constant g: m/s^2 )"
        LASSPentry $w.masses.g.e  {catch {set g [%W get]; reset_to_initial}}
        LASSPentry_set $w.masses.g.e $g
        pack $w.masses.g.l -side left
        pack $w.masses.g.e -side left -fill x -expand yes
        pack $w.masses.g -fill x -expand yes -pady 1

    pack $w.masses -fill x -expand yes -pady 1

    frame $w.algorithm
        label $w.algorithm.l -text "Algorithm"
        pack $w.algorithm.l -fill x -expand yes

        frame $w.algorithm.dt
        label $w.algorithm.dt.l \
                -text "Time step dt(s): (smaller: more accurate)\n"
        LASSPentry $w.algorithm.dt.e \
                {catch {set dt [%W get]}}
        LASSPentry_set $w.algorithm.dt.e $dt
        pack $w.algorithm.dt.l -side left
        pack $w.algorithm.dt.e \
                -side left -fill x -expand yes
        pack $w.algorithm.dt \
                 -fill x -expand yes -pady 1

	frame $w.algorithm.tmax
        label $w.algorithm.tmax.l -text "Time to Run (s):"
        LASSPentry $w.algorithm.tmax.e  \
                {catch {set tmax [%W get];
		 LASSPscale2_set .tM $tmax;
		 reset_to_initial}}
        LASSPentry_set $w.algorithm.tmax.e $tmax
        pack $w.algorithm.tmax.l -side left
        pack $w.algorithm.tmax.e -side left -fill x -expand yes
        pack $w.algorithm.tmax -fill x -expand yes -pady 1

    pack $w.algorithm -fill x -expand yes -pady 1

    label $w.warning -text "Close and Reopen to See Changes from Within Program"
    pack $w.warning -fill x -expand yes

}



############################## special bindings ########################
# bind . <Key-space> {.run invoke}	-- Interfered with typing in windows
# bind . <Key-i> {.init invoke}
bind . <Key-Escape> {.quit invoke}
