set SSS_PROG "jupiter"

# Debugging Tool
# LASSPinterp .interp


# tcl_precision: how many digits saved between snapshots

set tcl_precision 8

# time and max running time (storage for slider)
set t 0.
set tmax_store 1.

# for printing
set PostScript_mode color
set PostScript_file PostScript.ps

############## Global variables: shared with c-code via Tcl_LinkVar ##########

# Global masses
set EarthMass 1.
set JupiterMass 317.83
set SunMass 332830.

set RJ 5.20316

# G
# This should be G in units of AU / (EarthMass Years^2)
# G = 6.670 x 10^{-11} newton m^2 / kg^2 (Feynman volume I, 7.9)
#   From World Almanac 1990:
#     Earth <-> Sun = 92.9 million miles  (min 91.4, max 94.6)
#     Earth Mass = 6 sextillion, 588 quintillion short tons (ugh).
#     ...
#   Let's set our units so earth will go once around in one year
#   if JupiterMass = 0:
#
# G = 4 * Pi^2 / SunMass
   set Pi 3.14159265358979324
   set G [expr 4 * $Pi * $Pi / $SunMass];
# Frequency of Jupiter's Orbit (2 Pi / Jovian Year)
   set omega [expr sqrt($G * ($JupiterMass + $SunMass) / ($RJ*$RJ*$RJ))]
# Jovian Period = 11.862972 in this program

# Global initial conditions:

# Defaults not shared with C
set RxEarth_default 1.
set RyEarth_default 0.
set VxEarth_default 0.
set VyEarth_default [expr double(2*3.1415926535897931160)]

set RxEarth0 $RxEarth_default
set RyEarth0 $RyEarth_default
set VxEarth0 $VxEarth_default
set VyEarth0 $VyEarth_default

set t0 0.

# Energy in rotating reference frame (should be conserved)
# Energy set in Jupiter.c
# energy copied from Energy so textvariable will be updated...

set Energy 0
set energy $Energy

# These global variables are set later, after .planets is set up
# center_x, center_y in pixels,
# expand = ratio pixels/AU

# Not shared with C
# view_left, view_top is upper left hand in AU
# view_right, view_bottom is lower right in AU

# Size to see Jupiter's orbit: minimum size_view is 5 AU
# not shared with C
set size_view 5.

# Global smooth, view
# smooth true for Bezier, false for lines
# view is trajectory, animation, floquet, poincare, or poincare_rotating
# v_mult, theta_Poincare are for poincare_rotating

# error_max, dxsav are for rk_variable
set error_max 1.e-7
set dxsav 0.01

# running - whether planets are currently moving
# runinc - how long to run between updates (longer - faster, more jerky)
set running 0
set runinc 1.0

set v_mult 0.5
set theta_Poincare 0.

set smooth false
set view animation

# Global Earth_color, Jupiter_color, Sun_color
set Earth_color green
set Jupiter_color orange
set Sun_color yellow

# Global Earth_radius, Jupiter_radius, Sun_radius, dot_radius
set Earth_radius 0.1
set Jupiter_radius .3
set Sun_radius .6
set dot_radius .02



# Set up color wheel for dots

########################### the color wheel ##########################
# source Jupiter_ColorWheel.tcl

set color_wheel(0) pink
set color_wheel(1) orange
set color_wheel(2) yellow
set color_wheel(3) green
set color_wheel(4) skyblue
set color_wheel(5) cyan
set color_wheel(6) violet

set color_index 0
set color $color_wheel($color_index)
set color_max 6

proc increment_color {} {
    global color color_index color_max color_wheel
    if {$color_index == $color_max} {
        set color_index 0;
    } else {
        incr color_index
    }
    set color $color_wheel($color_index)
}


########## create & init main window #############

frame .planet_frame
frame .planet_menu
menubutton .view -text "View" -menu .view.menu -relief raised
menu .view.menu
.view.menu add radiobutton -label "Animation" \
        -variable view -value animation \
        -command {
                initialize_display
        }
.view.menu add radiobutton -label "Earth's Trajectory" \
        -variable view -value trajectory \
        -command {
                initialize_display
        }
.view.menu add radiobutton -label "Poincare (Rotating Ref. Frame)" \
        -variable view -value poincare_rotating \
        -command {
                initialize_display
                }

pack .view -in .planet_menu -side left -padx 2

button .about -text "About Jupiter" -command AboutJupiter
pack .about -in .planet_menu -side left -padx 2

LASSPpresets .presets presetsave
pack .presets -in .planet_menu -side left -padx 3

button .help -text "Help" -command {LASSPhelp2 .t2 [file join $SSS_DIR lib/jupiter/jupiter.htm]}
pack .help -in .planet_menu -side left -padx 3

proc AboutJupiter {} {
set w [toplevel .aboutJupiter]

message $w.msg -justify center -aspect 1000 -text \
"Jupiter 0.8
James Sethna
sethna@lassp.cornell.edu
http://www.lassp.cornell.edu/sethna/sethna.html
utilities developed with the assistance
of the Sloan Foundation and the SSS project
Copywrite May, 1997"

button $w.close -text Close -command "destroy $w"

pack $w.msg
pack $w.close
}

frame .time
label .time.l -text "Time"
label .time.v -textvariable t
pack .time.l; pack .time.v
pack .time -in .planet_menu -side right -padx 2
set warn ""
frame .warn
label .warn.v -textvariable warn
pack .warn.v
pack .warn -in .planet_menu -side right -padx 2
# frame .energy
# label .energy.l -text "Energy"
# label .energy.v -textvariable energy
# pack .energy.l .energy.v;
# pack .energy -in .planet_menu -side right -padx 2
pack .planet_menu -in .planet_frame -side top -fill x -pady 1

canvas .planets -relief sunken -background black
pack .planets -in .planet_frame -side top -fill both -expand yes -pady 1
pack .planet_frame -side top -fill both -expand yes -pady 1

############## Global variables for zooming set here #########################

set zooming 0	;# To tell if animation is currently zooming

#Solar system center in pixel coordinates:
set center_x [expr double([winfo reqwidth .planets])/2.]
set center_y [expr double([winfo reqheight .planets])/2.]
#Expansion factor AU -> Pixels
# view_left, view_top is upper left hand
# view_right, view_bottom is lower right

if {$center_x > $center_y} {
    set expand [expr $center_y/$size_view]
    set view_left [expr -($center_x/$center_y)*$size_view]
    set view_right [expr ($center_x/$center_y)*$size_view]
    set view_top [expr -$size_view]   ;# Top is minimum y
    set view_bottom [expr $size_view]
} else {
    set expand [expr $center_x/$size_view]
    set view_left [expr -$size_view]
    set view_right [expr $size_view]
    set view_top [expr -($center_y/$center_x)*$size_view]
    set view_bottom [expr ($center_y/$center_x)*$size_view]
}

########################### the presetsave procedure ##########################
proc presetsave {preset_file} {
    global t t0 PostScript_mode PostScript_file
    global RxEarth0 RyEarth0 VxEarth0 VyEarth0 RJ
    global RxEarth_default RyEarth_default VxEarth_default VyEarth_default
    global energy Energy center_x center_y size_view expand
    global smooth view error_max dxsav v_mult theta_Poincare
    global view_left view_right view_top view_bottom
    global Earth_color Jupiter_color Sun_color
    global Earth_radius Jupiter_radius Sun_radius dot_radius
    global .jM .tM EarthMass JupiterMass SunMass tmax runinc
    global omega G
    global tcl_precision

    #sliders
    foreach s {.jM .tM} {
        set val [LASSPscale2_configure $s -scrollinc]
        puts $preset_file "LASSPscale2_configure $s -scrollinc $val"
        set val [LASSPscale2_configure $s -scrollmin]
        puts $preset_file "set ${s}(-scrollmin) [set ${s}(-scrollmin)]"
        set val [LASSPscale2_configure $s -scrollmax]
        puts $preset_file "set ${s}(-scrollmax) [set ${s}(-scrollmax)]"
        set val [LASSPscale2_configure $s -scrolltype]
        puts $preset_file "set ${s}(-scrolltype) [set ${s}(-scrolltype)]"
        set val [LASSPscale2_configure $s -font]
        puts $preset_file "LASSPscale2_configure $s -font $val"
    }
    puts $preset_file "LASSPscale2_set .jM $JupiterMass"
    puts $preset_file "LASSPscale2_set .tM $tmax"

    puts $preset_file "set t $t"
    puts $preset_file "set t0 $t0"
    puts $preset_file "set PostScript_mode $PostScript_mode"
    puts $preset_file "set PostScript_file $PostScript_file"
    puts $preset_file "set RxEarth_default $RxEarth_default"
    puts $preset_file "set RyEarth_default $RyEarth_default"
    puts $preset_file "set VxEarth_default $VxEarth_default"
    puts $preset_file "set VyEarth_default $VyEarth_default"
    puts $preset_file "set RxEarth0 $RxEarth0"
    puts $preset_file "set RyEarth0 $RyEarth0"
    puts $preset_file "set VxEarth0 $VxEarth0"
    puts $preset_file "set VyEarth0 $VyEarth0"
    puts $preset_file "set omega $omega"
    puts $preset_file "set G $G"
    puts $preset_file "set RJ $RJ"
    puts $preset_file "set EarthMass $EarthMass"
    puts $preset_file "set SunMass $SunMass"
    puts $preset_file "set energy $energy"
    puts $preset_file "set Energy $Energy"
    puts $preset_file "set center_x $center_x"
    puts $preset_file "set center_y $center_y"
    puts $preset_file "set size_view $size_view"
    puts $preset_file "set view_left $view_left"
    puts $preset_file "set view_right $view_right"
    puts $preset_file "set view_top $view_top"
    puts $preset_file "set view_bottom $view_bottom"
    puts $preset_file "set expand $expand"
    puts $preset_file "set smooth $smooth"
    puts $preset_file "set view $view"
    puts $preset_file "set error_max $error_max"
    puts $preset_file "set dxsav $dxsav"
    puts $preset_file "set tcl_precision $tcl_precision"
    puts $preset_file "set v_mult $v_mult"
    puts $preset_file "set theta_Poincare $theta_Poincare"
    puts $preset_file "set runinc $runinc"
    puts $preset_file "set Earth_color $Earth_color"
    puts $preset_file "set Jupiter_color $Jupiter_color"
    puts $preset_file "set Sun_color $Sun_color"
    puts $preset_file "set Earth_radius $Earth_radius"
    puts $preset_file "set Jupiter_radius $Jupiter_radius"
    puts $preset_file "set Sun_radius $Sun_radius"
    puts $preset_file "set dot_radius $dot_radius"

    puts $preset_file ".planets delete all"
    puts $preset_file "reset"
    puts $preset_file "initialize_display"

}


########################## create a configure window ##########################
# source Jupiter_Configure.tcl
# configure radius & color for Earth, Jupiter, and Sun
#

proc configure {} {
    global RxEarth_default RyEarth_default VxEarth_default VyEarth_default
    global RxEarth0 RyEarth0 VxEarth0 VyEarth0
    global RJ omega G
    global Earth_color Jupiter_color Sun_color
    global Earth_radius Jupiter_radius Sun_radius dot_radius
    global smooth view error_max runinc dxsav steps v_mult theta_Poincare
    global EarthMass JupiterMass SunMass
    global t0
    global tcl_precision

# Child toplevel windows

    set child_toplevel_windows {
        colors_and_sizes
        initial
        masses
        algorithm
    }

# Reset toplevel windows if they exist
    if {[winfo exists .three_body_configure]} {
        foreach win [winfo children .three_body_configure] {
            destroy $win
        }
    } else {
        toplevel .three_body_configure
        wm title .three_body_configure "three body configure"
    }

    foreach win_top $child_toplevel_windows {
        if {[winfo exists .three_body_configure_$win_top]} {
            foreach win [winfo children .three_body_configure_$win_top] {
                destroy $win
            }
            configure_$win_top
        }
    }

    set w .three_body_configure

    frame $w.controls
    label $w.controls.l -text "Three Body Configure"
    button $w.controls.b -text Close -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    button $w.colors_and_sizes -text "Colors and Sizes" \
        -command configure_colors_and_sizes
    pack $w.colors_and_sizes -fill x -expand yes

    button $w.initial -text "Initial Conditions" \
        -command configure_initial
    pack $w.initial -fill x -expand yes

    button $w.masses -text "Masses and G" \
        -command configure_masses
    pack $w.masses -fill x -expand yes

    button $w.algorithms -text "Algorithm Settings" \
        -command configure_algorithm
    pack $w.algorithms -fill x -expand yes

    label $w.warning -text "Close and Reopen to See Changes from Within Program"
    pack $w.warning -fill x -expand yes

}

proc configure_colors_and_sizes {} {
    global Earth_color Jupiter_color Sun_color
    global Earth_radius Jupiter_radius Sun_radius dot_radius

    if {[winfo exists .three_body_configure_colors_and_sizes]} {
            foreach win \
                [winfo children .three_body_configure_colors_and_sizes] {
            destroy $win
        }
    } else {
        toplevel .three_body_configure_colors_and_sizes
        wm title .three_body_configure_colors_and_sizes \
                "configure colors and sizes"
    }
    set w .three_body_configure_colors_and_sizes

    frame $w.controls
    label $w.controls.l -text "Colors and Sizes"
    button $w.controls.b -text Close \
        -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.colors
        label $w.colors.l -text "Planet Colors"
        pack $w.colors.l -fill x -expand yes

        frame $w.colors.earth
        label $w.colors.earth.l -text "Earth Color:"
        LASSPentry $w.colors.earth.e  {catch {set Earth_color [%W get]}}
        LASSPentry_set $w.colors.earth.e $Earth_color
        pack $w.colors.earth.l -side left
        pack $w.colors.earth.e -side left -fill x -expand yes
        pack $w.colors.earth -fill x -expand yes -pady 1

        frame $w.colors.jupiter
        label $w.colors.jupiter.l -text "Jupiter Color:"
        LASSPentry $w.colors.jupiter.e  {catch {set Jupiter_color [%W get]}}
        LASSPentry_set $w.colors.jupiter.e $Jupiter_color
        pack $w.colors.jupiter.l -side left
        pack $w.colors.jupiter.e -side left -fill x -expand yes
        pack $w.colors.jupiter -fill x -expand yes -pady 1

        frame $w.colors.sun
        label $w.colors.sun.l -text "Sun Color:"
        LASSPentry $w.colors.sun.e  {catch {set Sun_color [%W get]}}
        LASSPentry_set $w.colors.sun.e $Sun_color
        pack $w.colors.sun.l -side left
        pack $w.colors.sun.e -side left -fill x -expand yes
        pack $w.colors.sun -fill x -expand yes -pady 1
    pack $w.colors -fill x -expand yes -pady 1

    frame $w.radius
        label $w.radius.l -text "Planet (and dot) Radii"
        pack $w.radius.l -fill x -expand yes

        frame $w.radius.dot
        label $w.radius.dot.l -text "Dot Size:"
        LASSPentry $w.radius.dot.e  {catch {set dot_radius [%W get]}}
        LASSPentry_set $w.radius.dot.e  $dot_radius
        pack $w.radius.dot.l -side left
        pack $w.radius.dot.e -side left -fill x -expand yes
        pack $w.radius.dot -fill x -expand yes -pady 1

        frame $w.radius.earth
        label $w.radius.earth.l -text "Earth Size:"
        LASSPentry $w.radius.earth.e  {catch {set Earth_radius [%W get]}}
        LASSPentry_set $w.radius.earth.e $Earth_radius
        pack $w.radius.earth.l -side left
        pack $w.radius.earth.e -side left -fill x -expand yes
        pack $w.radius.earth -fill x -expand yes -pady 1

        frame $w.radius.jupiter
        label $w.radius.jupiter.l -text "Jupiter Size:"
        LASSPentry $w.radius.jupiter.e  {catch {set Jupiter_radius [%W get]}}
        LASSPentry_set $w.radius.jupiter.e $Jupiter_radius
        pack $w.radius.jupiter.l -side left
        pack $w.radius.jupiter.e -side left -fill x -expand yes
        pack $w.radius.jupiter -fill x -expand yes -pady 1

        frame $w.radius.sun
        label $w.radius.sun.l -text "Sun Size:"
        LASSPentry $w.radius.sun.e  {catch {set Sun_radius [%W get]}}
        LASSPentry_set $w.radius.sun.e $Sun_radius
        pack $w.radius.sun.l -side left
        pack $w.radius.sun.e -side left -fill x -expand yes
        pack $w.radius.sun -fill x -expand yes -pady 1
    pack $w.radius -fill x -expand yes -pady 1

}

proc configure_initial {} {
    global RxEarth_default RyEarth_default VxEarth_default VyEarth_default
    global RxEarth0 RyEarth0 VxEarth0 VyEarth0
    global RJ omega
    global t0

    if {[winfo exists .three_body_configure_initial]} {
            foreach win [winfo children .three_body_configure_initial] {
            destroy $win
        }
    } else {
        toplevel .three_body_configure_initial
        wm title .three_body_configure_initial "Initial Conditions"
    }
    set w .three_body_configure_initial

    frame $w.controls
    label $w.controls.l -text "Initial Conditions"
    button $w.controls.b -text Close -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1


    frame $w.initial_conditions

        frame $w.initial_conditions.rxEarth0
        label $w.initial_conditions.rxEarth0.l -text "Earth X-Coordinate (AU):"
        LASSPentry $w.initial_conditions.rxEarth0.e  \
                {catch {set RxEarth0 [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.rxEarth0.e $RxEarth0
        pack $w.initial_conditions.rxEarth0.l -side left
        pack $w.initial_conditions.rxEarth0.e -side left -fill x -expand yes
        pack $w.initial_conditions.rxEarth0 -fill x -expand yes -pady 1

        frame $w.initial_conditions.ryEarth0
        label $w.initial_conditions.ryEarth0.l -text "Earth Y-Coordinate (AU):"
        LASSPentry $w.initial_conditions.ryEarth0.e  \
                {catch {set RyEarth0 [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.ryEarth0.e $RyEarth0
        pack $w.initial_conditions.ryEarth0.l -side left
        pack $w.initial_conditions.ryEarth0.e -side left -fill x -expand yes
        pack $w.initial_conditions.ryEarth0 -fill x -expand yes -pady 1

        frame $w.initial_conditions.vxEarth0
        label $w.initial_conditions.vxEarth0.l \
                -text "Earth X-Velocity (AU/year):"
        LASSPentry $w.initial_conditions.vxEarth0.e  \
                {catch {set VxEarth0 [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.vxEarth0.e $VxEarth0
        pack $w.initial_conditions.vxEarth0.l -side left
        pack $w.initial_conditions.vxEarth0.e -side left -fill x -expand yes
        pack $w.initial_conditions.vxEarth0 -fill x -expand yes -pady 1

        frame $w.initial_conditions.vyEarth0
        label $w.initial_conditions.vyEarth0.l \
                -text "Earth Y-Velocity (AU/year):"
        LASSPentry $w.initial_conditions.vyEarth0.e  \
                {catch {set VyEarth0 [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.vyEarth0.e $VyEarth0
        pack $w.initial_conditions.vyEarth0.l -side left
        pack $w.initial_conditions.vyEarth0.e -side left -fill x -expand yes
        pack $w.initial_conditions.vyEarth0 -fill x -expand yes -pady 1

        frame $w.initial_conditions.rJ
        label $w.initial_conditions.rJ.l \
                -text "Jupiter Radius (AU) (doesn't change period?):"
        LASSPentry $w.initial_conditions.rJ.e  \
                {catch {set RJ [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.rJ.e $RJ
        pack $w.initial_conditions.rJ.l -side left
        pack $w.initial_conditions.rJ.e -side left -fill x -expand yes
        pack $w.initial_conditions.rJ -fill x -expand yes -pady 1

        frame $w.initial_conditions.omega
        label $w.initial_conditions.omega.l \
                -text "Jupiter Frequency (2 Pi / Jovian Year):"
        LASSPentry $w.initial_conditions.omega.e  \
                {catch {set omega [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.omega.e $omega
        pack $w.initial_conditions.omega.l -side left
        pack $w.initial_conditions.omega.e -side left -fill x -expand yes
        pack $w.initial_conditions.omega -fill x -expand yes -pady 1

        frame $w.initial_conditions.t
        label $w.initial_conditions.t.l \
                -text "Initial time t0 (varies starting Jovian position):"
        LASSPentry $w.initial_conditions.t.e  \
                {catch {set t0 [%W get]; reset}}
        LASSPentry_set $w.initial_conditions.t.e $t0
        pack $w.initial_conditions.t.l -side left
        pack $w.initial_conditions.t.e -side left -fill x -expand yes
        pack $w.initial_conditions.t -fill x -expand yes -pady 1

    pack $w.initial_conditions -fill x -expand yes -pady 1


    frame $w.init_cond_default
        label $w.init_cond_default.l -text "Reset Initial Conditions"
        pack $w.init_cond_default.l -fill x -expand yes

        frame $w.init_cond_default.rxEarth_default
        label $w.init_cond_default.rxEarth_default.l \
                                -text "Default Earth X-Coordinate (AU):"
        LASSPentry $w.init_cond_default.rxEarth_default.e  \
                {catch {set RxEarth_default [%W get]; reset}}
        LASSPentry_set $w.init_cond_default.rxEarth_default.e $RxEarth_default
        pack $w.init_cond_default.rxEarth_default.l -side left
        pack $w.init_cond_default.rxEarth_default.e -side left -fill x \
                                                        -expand yes
        pack $w.init_cond_default.rxEarth_default -fill x -expand yes -pady 1

        frame $w.init_cond_default.ryEarth_default
        label $w.init_cond_default.ryEarth_default.l \
                                -text "Default Earth Y-Coordinate (AU):"
        LASSPentry $w.init_cond_default.ryEarth_default.e  \
                {catch {set RyEarth_default [%W get]; reset}}
        LASSPentry_set $w.init_cond_default.ryEarth_default.e $RyEarth_default
        pack $w.init_cond_default.ryEarth_default.l -side left
        pack $w.init_cond_default.ryEarth_default.e -side left -fill x \
                                                        -expand yes
        pack $w.init_cond_default.ryEarth_default -fill x -expand yes -pady 1

        frame $w.init_cond_default.vxEarth_default
        label $w.init_cond_default.vxEarth_default.l \
                -text "Default Earth X-Velocity (AU/year):"
        LASSPentry $w.init_cond_default.vxEarth_default.e \
                {catch {set VxEarth_default [%W get]; reset}}
        LASSPentry_set $w.init_cond_default.vxEarth_default.e $VxEarth_default
        pack $w.init_cond_default.vxEarth_default.l -side left
        pack $w.init_cond_default.vxEarth_default.e -side left -fill x \
                                                        -expand yes
        pack $w.init_cond_default.vxEarth_default -fill x -expand yes -pady 1

        frame $w.init_cond_default.vyEarth_default
        label $w.init_cond_default.vyEarth_default.l \
                -text "Default Earth Y-Velocity (AU/year):"
        LASSPentry $w.init_cond_default.vyEarth_default.e \
                {catch {set VyEarth_default [%W get]; reset}}
        LASSPentry_set $w.init_cond_default.vyEarth_default.e $VyEarth_default
        pack $w.init_cond_default.vyEarth_default.l -side left
        pack $w.init_cond_default.vyEarth_default.e -side left -fill x \
                                                        -expand yes
        pack $w.init_cond_default.vyEarth_default -fill x -expand yes -pady 1

    pack $w.init_cond_default -fill x -expand yes -pady 1

}

proc configure_masses {} {
    global EarthMass JupiterMass SunMass G

    if {[winfo exists .three_body_configure_masses]} {
            foreach win [winfo children .three_body_configure_masses] {
            destroy $win
        }
    } else {
        toplevel .three_body_configure_masses
        wm title .three_body_configure_masses "Masses (AU) and G"
    }
    set w .three_body_configure_masses

    frame $w.controls
    label $w.controls.l -text "Masses and G"
    button $w.controls.b -text Close \
        -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.masses
        frame $w.masses.earth
        label $w.masses.earth.l -text "Earth Mass: (irrelevant)"
        LASSPentry $w.masses.earth.e {catch {set EarthMass [%W get]; reset}}
        LASSPentry_set $w.masses.earth.e $EarthMass
        pack $w.masses.earth.l -side left
        pack $w.masses.earth.e -side left -fill x -expand yes
        pack $w.masses.earth -fill x -expand yes -pady 1

        frame $w.masses.jupiter
        label $w.masses.jupiter.l -text "Jupiter Mass:"
        LASSPentry $w.masses.jupiter.e  \
            {catch {set JupiterMass [%W get];
                    LASSPscale2_set .jM $JupiterMass;
                    reset}}
        LASSPentry_set $w.masses.jupiter.e $JupiterMass
        pack $w.masses.jupiter.l -side left
        pack $w.masses.jupiter.e -side left -fill x -expand yes
        pack $w.masses.jupiter -fill x -expand yes -pady 1

        frame $w.masses.sun
        label $w.masses.sun.l -text "Sun Mass:"
        LASSPentry $w.masses.sun.e {catch {set SunMass [%W get]; reset}}
        LASSPentry_set $w.masses.sun.e $SunMass
        pack $w.masses.sun.l -side left
        pack $w.masses.sun.e -side left -fill x -expand yes
        pack $w.masses.sun -fill x -expand yes -pady 1

        frame $w.masses.g
        frame $w.masses.g_top
        label $w.masses.g_top.l \
                -text "Gravitational Constant G: AU / (EarthMass Years^2)"
        pack $w.masses.g_top.l -side left
        pack $w.masses.g_top -fill x -expand yes -pady 2
        label $w.masses.g.l \
                -text "(Set so earth year = 1 when MJ = 0)"
        LASSPentry $w.masses.g.e  \
                {catch {set G [%W get]; reset}}
        LASSPentry_set $w.masses.g.e $G
        pack $w.masses.g.l -side left
        pack $w.masses.g.e -side left -fill x -expand yes
        pack $w.masses.g -fill x -expand yes -pady 1

    pack $w.masses -fill x -expand yes -pady 1

}

proc configure_algorithm {} {
    global smooth view algorithm error_max runinc dxsav v_mult theta_Poincare
    global tcl_precision

    if {[winfo exists .three_body_configure_algorithm]} {
            foreach win [winfo children .three_body_configure_algorithm] {
            destroy $win
        }
    } else {
        toplevel .three_body_configure_algorithm
        wm title .three_body_configure_algorithm "algorithm settings"
    }
    set w .three_body_configure_algorithm

    frame $w.controls
    label $w.controls.l -text "Algorithm Settings"
    button $w.controls.b -text Close \
        -command "destroy $w"

    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.algorithm

        frame $w.algorithm.runinc
        label $w.algorithm.runinc.l \
                -text "Run Increment (smaller=less jerky):\n"
        LASSPentry $w.algorithm.runinc.e \
                {catch {set runinc [%W get]}}
        LASSPentry_set $w.algorithm.runinc.e $runinc
        pack $w.algorithm.runinc.l -side left
        pack $w.algorithm.runinc.e \
                -side left -fill x -expand yes
        pack $w.algorithm.runinc \
                 -fill x -expand yes -pady 1

        frame $w.algorithm.tcl_precision
        label $w.algorithm.tcl_precision.l \
                -text "tcl Precision: mostly for plotting expanded regions\n"
        LASSPentry $w.algorithm.tcl_precision.e \
                {catch {set tcl_precision [%W get]}}
        LASSPentry_set $w.algorithm.tcl_precision.e $tcl_precision
        pack $w.algorithm.tcl_precision.l -side left
        pack $w.algorithm.tcl_precision.e \
                -side left -fill x -expand yes
        pack $w.algorithm.tcl_precision \
                 -fill x -expand yes -pady 1

        frame $w.algorithm.smooth
        label $w.algorithm.smooth.l \
                -text "Smoothing: for trajectory\n"
        LASSPentry $w.algorithm.smooth.e \
                {catch {set smooth [%W get]}}
        LASSPentry_set $w.algorithm.smooth.e $smooth
        pack $w.algorithm.smooth.l -side left
        pack $w.algorithm.smooth.e \
                -side left -fill x -expand yes
        pack $w.algorithm.smooth \
                 -fill x -expand yes -pady 1

        frame $w.algorithm.rk_variable
            frame $w.algorithm.rk_variable.error_max
            label $w.algorithm.rk_variable.error_max.l \
                -text "Error Maximum:"
            LASSPentry $w.algorithm.rk_variable.error_max.e \
                {catch {set error_max [%W get]}}
            LASSPentry_set $w.algorithm.rk_variable.error_max.e $error_max
            pack $w.algorithm.rk_variable.error_max.l -side left
            pack $w.algorithm.rk_variable.error_max.e \
                -side left -fill x -expand yes
            pack $w.algorithm.rk_variable.error_max \
                 -fill x -expand yes -pady 1

            frame $w.algorithm.rk_variable.dxsav
            label $w.algorithm.rk_variable.dxsav.l \
                -text "Display Step Minimum Size:"
            LASSPentry $w.algorithm.rk_variable.dxsav.e \
                {catch {set dxsav [%W get]}}
            LASSPentry_set $w.algorithm.rk_variable.dxsav.e $dxsav
            pack $w.algorithm.rk_variable.dxsav.l -side left
            pack $w.algorithm.rk_variable.dxsav.e \
                -side left -fill x -expand yes
            pack $w.algorithm.rk_variable.dxsav \
                 -fill x -expand yes -pady 1

            frame $w.algorithm.rk_variable.v_mult
            label $w.algorithm.rk_variable.v_mult.l \
                -text "Velocity Multiplier for Poincare Section:"
            LASSPentry $w.algorithm.rk_variable.v_mult.e \
                {catch {set v_mult [%W get]}}
            LASSPentry_set $w.algorithm.rk_variable.v_mult.e $v_mult
            pack $w.algorithm.rk_variable.v_mult.l -side left
            pack $w.algorithm.rk_variable.v_mult.e \
                -side left -fill x -expand yes
            pack $w.algorithm.rk_variable.v_mult \
                 -fill x -expand yes -pady 1

            frame $w.algorithm.rk_variable.theta_Poincare
            label $w.algorithm.rk_variable.theta_Poincare.l \
                -text "Angular Lag for Poincare Section:"
            LASSPentry $w.algorithm.rk_variable.theta_Poincare.e \
                {catch {set theta_Poincare [%W get]}}
            LASSPentry_set $w.algorithm.rk_variable.theta_Poincare.e \
                        $theta_Poincare
            pack $w.algorithm.rk_variable.theta_Poincare.l -side left
            pack $w.algorithm.rk_variable.theta_Poincare.e \
                -side left -fill x -expand yes
            pack $w.algorithm.rk_variable.theta_Poincare \
                 -fill x -expand yes -pady 1


        pack $w.algorithm.rk_variable -fill x -expand yes -pady 1
    pack $w.algorithm -fill x -expand yes -pady 1

}

######################################################################


proc reset_to_defaults {} {
    global t t0 energy Energy view
    global RxEarth0 RyEarth0 VxEarth0 VyEarth0
    global RxEarth_default RyEarth_default VxEarth_default VyEarth_default

    stop
    increment_color
    .run configure -text "Run"
    set t $t0
    set RxEarth0 $RxEarth_default
    set RyEarth0 $RyEarth_default
    set VxEarth0 $VxEarth_default
    set VyEarth0 $VyEarth_default
    Reset_c
    set energy $Energy

    if {$view == "animation"} initialize_display
}

proc reset {} {
    global t t0 energy Energy view
    global RxEarth0 RyEarth0 VxEarth0 VyEarth0
    global RxEarth_default RyEarth_default VxEarth_default VyEarth_default

    stop
    increment_color
    .run configure -text "Run"
    set t $t0
    Reset_c
    set energy $Energy

    if {$view == "animation"} initialize_display
}

#################### Print PostScript file for Jupiter #################
# Set up PostScript file output for canvas window
# sets up command print {}
# source Jupiter_Print.tcl

proc print {} {
    global .planets PostScript_mode PostScript_file

    if {[winfo exists .print]} {
            foreach win [winfo children .print] {
            destroy $win
        }
    } else {
        toplevel .print
        wm title .print "Print"
    }

    set w .print

    frame $w.controls
    label $w.controls.l -text "Save PostScript File"
    button $w.controls.b -text Close -command "destroy .print"
    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1

    frame $w.info
        label $w.info.l -text "Save to PostScript File"
        pack $w.info.l -fill x -expand yes

        frame $w.info.file
        label $w.info.file.l -text "File Name:"
        LASSPentry $w.info.file.e  {catch {set PostScript_file [%W get]}}
        pack $w.info.file.l -side left
        pack $w.info.file.e -side left -fill x -expand yes
        $w.info.file.e insert 0 $PostScript_file
        pack $w.info.file -fill x -expand yes -pady 1

        frame $w.info.mode
        label $w.info.mode.l -text "Mode (color, gray, or mono):"
        LASSPentry $w.info.mode.e  {catch {set PostScript_mode [%W get]}}
        pack $w.info.mode.l -side left
        pack $w.info.mode.e -side left -fill x -expand yes
        $w.info.mode.e insert 0 $PostScript_mode
        pack $w.info.mode -fill x -expand yes -pady 1

        button $w.info.save -text "Save File" \
            -command {
                .planets postscript -file $PostScript_file \
                -colormode $PostScript_mode \
                -pagewidth 6.5i -pageheight 8.5i -rotate true
                foreach win [winfo children .print] { destroy $win }
                destroy .print
                }
        pack $w.info.save -fill x -expand yes -pady 1
    pack $w.info -fill x -expand yes -pady 1

}

########################## Select Initial ############################
# Uses button 1 to select initial conditions for various display options #
# source Jupiter_Select_Initial.tcl

bind .planets <Button1-Motion> {
    if {($view == "animation")||($view == "trajectory")} {
        set xF %x
        set yF %y
        set cor [lreplace [.planets coords velocity] 2 3 %x %y]
        eval ".planets coords velocity $cor"
    }
}
bind .planets <ButtonRelease-1> {
    if {($view == "animation")||($view == "trajectory")} {
        .planets delete velocity
        set RxEarth0 [expr ($xI - $center_x)/$expand]
        set RyEarth0 [expr ($yI - $center_y)/$expand]
        set VxEarth0 [expr ($xF - $xI)*$omega/($expand*$v_mult)]
        set VyEarth0 [expr ($yF - $yI)*$omega/($expand*$v_mult)]
        reset
        if {$view == "animation"} {initialize_display}
        .run invoke
    }
}

bind .planets <ButtonPress-1> {
    if {($view == "animation")||($view == "trajectory")} {
        .planets create line %x %y %x %y -tags velocity -arrow last -fill orange
        set xI %x
        set yI %y
        set xF %x
        set yF %y
    }
    if {$view == "poincare_rotating"} {
        # Set R_parallel, V_parallel with mouse clicks
        # Find V_perp to keep on same energy surface
        # Doesn't do anything if V_parallel too large for solution
        reset
        set R_parallel [expr (%x - $center_x)/$expand]
        set R_perp 0.
        # don't set RxEarth0, RyEarth0 until discriminant checked
        set RxEarth [expr $R_parallel * cos($omega*$t + $theta_Poincare)]
        set RyEarth [expr $R_parallel * sin($omega*$t + $theta_Poincare)]

        set RxJupiter [expr ($RJ*$SunMass/($JupiterMass+$SunMass)) \
                        * cos($omega*$t)]
        set RyJupiter [expr ($RJ*$SunMass/($JupiterMass+$SunMass)) \
                        * sin($omega*$t)]
        set RxSun [expr - ($RJ*$JupiterMass/($JupiterMass+$SunMass)) \
                        * cos($omega*$t)]
        set RySun [expr - ($RJ*$JupiterMass/($JupiterMass+$SunMass)) \
                        * sin($omega*$t)]
        set V_parallel [expr (%y - $center_y)*$omega/($v_mult*$expand)]
        set Potential_Energy \
                [expr - $G*$EarthMass*$JupiterMass      \
                    / sqrt(($RxJupiter-$RxEarth) * ($RxJupiter-$RxEarth) \
                        +($RyJupiter-$RyEarth) * ($RyJupiter-$RyEarth)) \
                      - $G*$EarthMass*$SunMass  \
                    / sqrt(($RxSun-$RxEarth) * ($RxSun-$RxEarth)   \
                        +($RySun-$RyEarth) * ($RySun-$RyEarth))]
        set C [expr - $Energy + $Potential_Energy \
                        + 0.5*$EarthMass*$V_parallel*$V_parallel \
                        - $EarthMass*$omega*$V_parallel*$R_perp]
        set discriminant \
                    [expr $omega*$omega*$R_parallel*$R_parallel \
                                 -2*$C/$EarthMass]
        if {$discriminant > 0} {
            set V_perp  [expr $omega*$R_parallel + sqrt($discriminant)]
            set VxEarth0 [expr $V_parallel * cos($omega*$t + $theta_Poincare) \
                                   - $V_perp * sin($omega*$t + $theta_Poincare)]
            set VyEarth0 [expr $V_parallel * sin($omega*$t + $theta_Poincare) \
                                   + $V_perp * cos($omega*$t + $theta_Poincare)]
            set RxEarth0 $RxEarth
            set RyEarth0 $RyEarth
	    set warn ""
            reset
            .run invoke
        } else {
	    set warn "VERBOTEN"
        }
    }
}

######################################################################
# Zooming stuff, for Jupiter
# Zoom with Button-3 in canvas window

# Setup Zoom
proc setup_zoom {x y} {
    global .planets band_xI band_yI band_x1 band_y1 band_x2 band_y2
    global zooming

    .planets create rectangle $x $y $x $y -tags band -outline white
    set band_xI $x
    set band_yI $y
    set band_x1 $x
    set band_y1 $y
    set band_x2 $x
    set band_y2 $y

    set zooming 1
}

bind .planets <Control-ButtonPress-1> {
    setup_zoom %x %y
}
bind .planets <ButtonPress-3> {
    setup_zoom %x %y
}

# Expand zoom rubber band
proc zoom_band {x y} {
    global band_xF band_yF band_xI band_yI band_x1 band_y1 band_x2 band_y2
    global .planets

    set band_xF $x
    set band_yF $y
    if {$band_xF > $band_xI} {
        set band_x1 $band_xI; set band_x2 $band_xF
    } else {
        set band_x1 $band_xF; set band_x2 $band_xI
    }
    if {$band_yF > $band_yI} {
        set band_y1 $band_yI; set band_y2 $band_yF
    } else {
        set band_y1 $band_yF; set band_y2 $band_yI
    }
    .planets coords band $band_x1 $band_y1 $band_x2 $band_y2

}

bind .planets <Control-Button1-Motion> {
    zoom_band %x %y
}
bind .planets <Button3-Motion> {
    zoom_band %x %y
}

# Reset to original scale
proc reset_scale {} {
    global size_view expand center_x center_y
    set win_x [expr double([winfo width .planets])]
    set win_y [expr double([winfo height .planets])]
    if {$win_x > $win_y} {
        set A [expr $win_y/(2.*$size_view*$expand)]
    } else {
        set A [expr $win_x/(2.*$size_view*$expand)]
    }
    if {$A == 1} {
        set shift_x [expr ($win_x/2. - $center_x)]
        set shift_y [expr ($win_y/2. - $center_y)]
        .planets move all $shift_x $shift_y
        set center_x [expr $center_x + $shift_x]
        set center_y [expr $center_y + $shift_y]
    } else {
        set expand [expr $expand * $A]
        set x_expand [expr ($win_x/2. - $A*$center_x)/(1.-$A)]
        set y_expand [expr ($win_y/2. - $A*$center_y)/(1.-$A)]
        .planets scale all $x_expand $y_expand $A $A
        set center_x [expr $A*($center_x - $x_expand) + $x_expand]
        set center_y [expr $A*($center_y - $y_expand) + $y_expand]
    }
    set view_left [expr -$center_x/$expand]
    set view_right [expr ($win_x-$center_x)/$expand]
    set view_top [expr -$center_y/$expand]
    set view_bottom [expr ($win_y-$center_y)/$expand]
}

# Do zooming
proc do_zoom {} {
    global band band_x1 band_y1 band_x2 band_y2
    global .planets
    global win_x win_y center_x center_y expand
    global view_left view_right view_top view_bottom
    global zooming

    set win_x [expr double([winfo width .planets])]
    set win_y [expr double([winfo height .planets])]
    if {[expr ($band_x2 - $band_x1 == 0) || ($band_y2 - $band_y1 == 0)]} {
#
# Put back at starting size
#
        reset_scale
        .planets delete band
    } else {
#
# Blow up keeping aspect ratios: rescale by whatever's smaller
# Make center of window equal center of zoom box
#
        set Ax [expr $win_x/($band_x2 - $band_x1)]
        set Ay [expr $win_y/($band_y2 - $band_y1)]
        if {$Ax < $Ay} {set A $Ax} else {set A $Ay}
# Either move (if scale unchanged)
        if {$A == 1} {
            set shift_x [expr ($win_x - $band_x1 - $band_x2)/2.]
            set shift_y [expr ($win_y - $band_y1 - $band_y2)/2.]
            .planets move all $shift_x $shift_y
            set center_x [expr $center_x + $shift_x]
            set center_y [expr $center_y + $shift_y]
            .planets delete band
        } else {
# or scale by A, keeping point (x_expand, y_expand) fixed
            set expand [expr $expand * $A]
            set x_expand [expr ($win_x/2.-$A*($band_x1+$band_x2)/2.)/(1.-$A)]
            set y_expand [expr ($win_y/2.-$A*($band_y1+$band_y2)/2.)/(1.-$A)]
            .planets scale all $x_expand $y_expand $A $A
            set center_x [expr $A*($center_x - $x_expand) + $x_expand]
            set center_y [expr $A*($center_y - $y_expand) + $y_expand]
            .planets delete band
        }
        set view_left [expr -$center_x/$expand]
        set view_right [expr ($win_x-$center_x)/$expand]
        set view_top [expr -$center_y/$expand]
        set view_bottom [expr ($win_y-$center_y)/$expand]
    }
    set zooming 0
}

bind .planets <Control-ButtonRelease-1> {
    do_zoom
}

bind .planets <ButtonRelease-3> {
    do_zoom
}

bind .planets <Configure> {

    set win_x [expr double([winfo width .planets])]
    set win_y [expr double([winfo height .planets])]

    set expand_old $expand
    set expand_x [expr $win_x/($view_right - $view_left)]
    set expand_y [expr $win_y/($view_bottom - $view_top)]
    if {$expand_x > $expand_y} {
        set expand $expand_y
    } else {
        set expand $expand_x
    }
    set A [expr $expand/$expand_old]
    if {$A == 1} {
        if {$expand_x != $expand_y} {   ;# Equal -> No change???
            set shift_x [expr ($win_x/2.)*(1-$expand/$expand_x)]
            set shift_y [expr ($win_y/2.)*(1-$expand/$expand_y)]
            .planets move all $shift_x $shift_y
            set center_x [expr $center_x + $shift_x]
            set center_y [expr $center_y + $shift_y]
        }
    } else {
        if {$expand_x > $expand_y} {
            set right [expr $win_x/2+$A*$expand_old*($view_right-$view_left)/2]
            set bottom $win_y
        } else {
            set right $win_x
            set bottom [expr $win_y/2+$A*$expand_old*($view_bottom-$view_top)/2]
        }
        set x_expand [expr ($right - $A*($expand_old*$view_right+$center_x)) \
                                /(1.-$A)]
        set y_expand [expr ($bottom - $A*($expand_old*$view_bottom+$center_y)) \
                                /(1.-$A)]
        .planets scale all $x_expand $y_expand $A $A
        set center_x [expr $A*($center_x - $x_expand) + $x_expand]
        set center_y [expr $A*($center_y - $y_expand) + $y_expand]
    }
    set view_left [expr -$center_x/$expand]
    set view_right [expr ($win_x-$center_x)/$expand]
    set view_top [expr -$center_y/$expand]
    set view_bottom [expr ($win_y-$center_y)/$expand]

}


######################################################################

#############################  Buttons and Controls #########################

frame .controls
pack .controls -fill x

label .title -text "Jupiter, Earth, and Sun" -bd 2 -relief ridge
pack .title -in .controls -fill x -pady 3

LASSPscale2 .jM -text "Jupiter Mass: " -scrollmin 0 -scrollmax 100000 \
        -initial 317.83 -scrollinc 1 -type real \
        -var JupiterMass -test { {%v>=0} && {1>0} }

LASSPscale2 .tM -text "Time (Years): " -scrollmin 1 -scrollmax 10000 \
        -initial 3 -scrollinc 1 -type real -scrolltype log -var tmax \
        -test { {%v>=0} && {1>0} }

pack .jM -in .controls -fill x -pady 3
pack .tM -in .controls -fill x -pady 3

frame .buttons

proc stop {} {
    global tmax_store running .tM
    if {$running} {
        .run configure -text "Continue"
        LASSPscale2_set .tM $tmax_store
        set running 0
    }
}

proc initialize_display {} {
    global running t
    if {$running} {} else {Run $t [expr $t]}
}

button .run -text "Run" -width 10 -command {
    if {$running} {stop} else {
        .run configure -text "Stop"
        update
        set running 1
        set tmax_store $tmax
        Run $t [expr $t+$tmax]
        if {$running} {
            LASSPscale2_set .tM $tmax_store
            .run configure -text "Continue"
            set running 0
            update
        }
    }
}

button .configure -text "Configure" -command {configure}
button .clear -text "Clear" -command {.planets delete all}
button .reset -text "Reset" -command {reset_to_defaults}
button .print_it -text "Print" -command {print}
button .quit -text quit -command {exit 0}
pack .run -in .buttons -side left -padx 3
pack .clear -in .buttons -side left -padx 3
pack .reset -in .buttons -side left -padx 3
pack .configure -in .buttons -side left -padx 3
pack .print_it -in .buttons -side left -padx 3
pack .quit -in .buttons -side left -padx 3
pack .buttons -in .controls -pady 3


# Presets used to have long names: separate line...

frame .varyButtons
pack .varyButtons -in .controls -pady 1 -fill x

# Set earth to initial position, velocity...
set t 0.
Reset_c
set energy $Energy
initialize_display

############################## special bindings #######################3
bind . <Key-i> {.initialize invoke}
bind . <Key-Escape> {.quit invoke}

wm minsize . 1 1

